#!/bin/bash
#===============================================================================
#
#          FILE:  kernel-build-cross.sh
#
#         USAGE:  ./kernel-build-cross.sh
#
#   DESCRIPTION:  Build a Linux kernel from a specified source tree with a
#                 given cross-compiler.
#
#        AUTHOR:  EMAC-OE Development Team
#       COMPANY:  2008-2009 (c), EMAC, Inc.
#       VERSION:  1.0
#       CREATED:  03/05/2009
#      REVISION:  4
#===============================================================================

. environment.cfg.sh

if [ ! -x ${CROSS_COMPILE}gcc ]
then
    echo "Please define a valid CROSS_COMPILE variable!"
    exit 1
fi

create_kernel_image() {
    cd $OUTDIR
    mkdir -p Install/{boot,lib}
    make -j3 O=$OUTDIR ARCH=$ARCH CROSS_COMPILE=$CROSS_COMPILE INSTALL_MOD_PATH=Install/ modules_install || exit 1
    cp System.map Install/boot/System.map-${KERNELNAME}
    cp arch/$ARCH/boot/bzImage Install/boot/bzImage-${KERNELNAME}
    cd Install/boot
    ln -s System.map-${KERNELNAME} System.map
    ls -s bzImage-${KERNELNAME} bzImage
    cd $OUTDIR
    rm -rf Install/lib/modules/*/{build,source}
    cd Install
    ## Fakeroot is required to keep permissions on /boot and /lib on the board
    if [ "`which fakeroot`" == "" ]
    then
        echo "Fakeroot package required to create a valid tarball."
        exit 0
    else
        fakeroot tar zcvf ../kernel-${KERNELNAME}.tar.gz *
        cd ..
        echo "kernel-${KERNELNAME}.tar.gz created."
    fi
}

print_usage() {

    cat <<!USAGE!
Usage: kernel-build-cross.sh SOURCE_TREE config|build|upload|buildup [BUILD-SUFFIX] [IPADDR]
       -- Build a kernel in a directory seperate from the source tree using a cross-compiler --
       SOURCE-TREE:   Directory containing kernel source tree to build
       BUILD-SUFFIX:  Suffix to add to build directory (i.e. date tag as 20060108)
       IPADDR:        IP address to upload the compressed image to
       config:        Runs make menuconfig
       build:         Runs make to compile kernel and build package. Creates image in tarball
       upload:        Uploads image to given IPADDR
       buildup:       Builds and then uploads to given IPADDR
       !! Be sure to modify the script to use the correct cross-compiler before running !!
!USAGE!

}

if [[ "$1" == "" ]]
then
    print_usage
    exit 0
fi

SRCDIR=$1

for i in VERSION PATCHLEVEL SUBLEVEL EXTRAVERSION
do
    #NAME[j]=`grep ^$i $MAKEFILE | awk -F " " "{ print $3 };"`
    NAME[j]=`grep ^$i $SRCDIR/Makefile | cut -d' ' -f3`
    let j=j+1
    #echo i is $i
    #echo j is $j
done

KERNELNAME="${NAME[0]}.${NAME[1]}.${NAME[2]}${NAME[3]}"

OUTDIR=$(pwd)/build-${KERNELNAME}-$3

echo "creating directory" $OUTDIR

mkdir -p $OUTDIR

# copy the .config (if exists) from source directory
if [ -f $OUTDIR/.config ]
then
    echo "Default configuration found in build directory..."
elif [ -f $SRCDIR/.config ]
then
    cp $SRCDIR/.config $OUTDIR
    mv $SRCDIR/.config $SRCDIR/prevconfig
elif [ -f $SRCDIR/prevconfig ]
then
    cp $SRCDIR/prevconfig $OUTDIR/.config
else
    echo "Unable to find default configuration file!!!"
fi


case "$2" in

    config )
        make -C $SRCDIR O=$OUTDIR ARCH=$ARCH CROSS_COMPILE=$CROSS_COMPILE menuconfig
        ;;


    oldconfig )
        make -C $SRCDIR O=$OUTDIR ARCH=$ARCH CROSS_COMPILE=$CROSS_COMPILE  oldconfig
        ;;

    build )
        make -C $SRCDIR O=$OUTDIR ARCH=$ARCH CROSS_COMPILE=$CROSS_COMPILE bzImage || exit 1
        make -C $SRCDIR O=$OUTDIR ARCH=$ARCH CROSS_COMPILE=$CROSS_COMPILE  modules || exit 1
        create_kernel_image;
        ;;

    image)
	make -C $SRCDIR O=$OUTDIR ARCH=$ARCH CROSS_COMPILE=$CROSS_COMPILE bzImage || exit 1
	;;

    modules)
    	make -C $SRCDIR O=$OUTDIR ARCH=$ARCH CROSS_COMPILE=$CROSS_COMPILE modules || exit 1
	;;

    package )
        create_kernel_image;
        ;;

    buildup )
        make -C $SRCDIR O=$OUTDIR ARCH=$ARCH CROSS_COMPILE=$CROSS_COMPILE uImage || exit 1
        make -C $SRCDIR O=$OUTDIR ARCH=$ARCH CROSS_COMPILE=$CROSS_COMPILE modules || exit 1
        create_kernel_image || exit 1
        wput -B $OUTDIR/kernel-${KERNELNAME}.tar.gz ftp://root:emac_inc@$4/../kernel-${KERNELNAME}.tar.gz
        ;;

    upload )
        wput -B $OUTDIR/kernel-${KERNELNAME}.tar.gz ftp://root:emac_inc@$4/../kernel-${KERNELNAME}.tar.gz
        ;;


    * )
        print_usage
        ;;

esac

exit 0

